-- Created by Steve Stedman
--    http://SteveStedman.com
--    twitter:   @SqlEmt

RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG


USE Master;
GO

IF EXISTS(SELECT name 
            FROM sys.databases 
		   WHERE name = 'QueryTraining')
BEGIN
	ALTER DATABASE [QueryTraining] 
	SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [QueryTraining];
END
CREATE DATABASE [QueryTraining];
GO

USE [QueryTraining];


CREATE TABLE [dbo].[Inventory](
	[InventoryID] [INTEGER] IDENTITY(-2147483647,1) PRIMARY KEY,
	[Category] [VARCHAR](200), 
	[ItemSKU] [VARCHAR](200),
	[Description] [VARCHAR](4000),
	[QuantityOnHand] [INT],
	[UnitCost] DECIMAL(10,2),
	[UnitRetail] DECIMAL(10,2),
	[Archived] BIT
); 

INSERT INTO dbo.Inventory (Category, ItemSKU, Description, QuantityOnHand, UnitCost, UnitRetail, Archived)
VALUES
('SLEEPINGBAG', 'SB001-LG-RED', 'Sleeping Bag - Large - RED', 5, 5.75, 19.99, 0),
('SLEEPINGBAG', 'SB001-LG-GRN', 'Sleeping Bag - Large - GREEN', 13, 5.75, 14.99, 1),
('SLEEPINGBAG', 'SB001-LG-BLK', 'Sleeping Bag - Large - BLACK', 1, 5.75, 19.99, 0),
('TENT', 'TT001-10-CMO', 'Tent - 10 Man - CAMO', 5, 10.00, 99.95, 0),
('AXE', 'AX013-HT-BLK', 'Axe - Hatchet - Black', 35, 4.99, 24.95, 0),
('FOOD', 'MR005-1S-SPG', 'MRE - 1 Serve - Spaghetti', 5, 0.50, 7.99, 0);
-------------------------------------------------------------------------

USE [QueryTraining];
GO
----------------------------------------------------------------



 --How can you get to the items WITH the lowest 5 UnitRetail prices?
SELECT *,
	   RANK() OVER(ORDER BY i.UnitRetail ASC) AS rankUnitCost
  FROM dbo.Inventory AS i
 WHERE (RANK() OVER(ORDER BY i.UnitRetail ASC)) <= 5;


SELECT *,
	   RANK() OVER(ORDER BY i.UnitRetail ASC) AS rankUnitCost
  FROM dbo.Inventory AS i
HAVING (RANK() OVER(ORDER BY i.UnitRetail ASC)) <= 5;


 --use a sub query  AKA.... A Derived Table Query
 SELECT * 
   FROM (

		   SELECT *,
				  RANK() OVER(ORDER BY i.UnitRetail ASC) AS rankUnitCost
		   FROM dbo.Inventory AS i

		) AS derivedTable
  WHERE derivedTable.rankUnitCost <= 5;




-- of the 5 cheapest products, show me those that are 
--     priced above average of the 5 cheapest

-- nested derived tables
SELECT * 
  FROM (

	SELECT *, AVG(derivedTable.UnitRetail) OVER() AS averageCheapest
	  FROM (

			   SELECT *,
					  RANK() OVER(ORDER BY i.UnitRetail ASC) AS rankUnitCost
			   FROM dbo.Inventory AS i

			) AS derivedTable
	  WHERE derivedTable.rankUnitCost <= 5
	
	  ) AS secondDerivedTable
 WHERE secondDerivedTable.UnitRetail > secondDerivedTable.averageCheapest;



-- in the where clause

SELECT *,
		RANK() OVER(ORDER BY i.UnitRetail ASC) AS rankUnitCost
FROM dbo.Inventory AS i;


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SELECT *,
		RANK() OVER(ORDER BY i.UnitRetail ASC) AS rankUnitCost
FROM dbo.Inventory AS i
WHERE rankUnitCost > 
	(SELECT DISTINCT AVG(UnitRank) OVER()
       FROM (
			 SELECT RANK() OVER(ORDER BY UnitRetail ASC) AS UnitRank 
               FROM dbo.Inventory
			 ) AS t)




-- in the SELECT columns list.


SELECT *,

	   (SELECT DISTINCT AVG(UnitRank) OVER()
          FROM (

			 SELECT RANK() OVER(ORDER BY UnitRetail ASC) AS UnitRank 
               FROM dbo.Inventory

			 ) AS t)
			  
			 AS derivedTableColumn
FROM dbo.Inventory AS i






